(function(d){d['cs']=Object.assign(d['cs']||{},{a:"Soubor nelze nahrát:",b:"Citace",c:"Kurzíva",d:"Tučné",e:"ovládací prvek obrázku",f:"Obrázek v plné velikosti",g:"Postranní obrázek",h:"Obrázek zarovnaný vlevo",i:"Obrázek zarovnaný na střed",j:"Obrázek zarovnaný vpravo",k:"Zadejte popis obrázku",l:"Vložit obrázek",m:"Odkaz",n:"Nahrání selhalo",o:"Zvolte nadpis",p:"Nadpis",q:"Číslování",r:"Odrážky",s:"Změnit alternativní text obrázku",t:"Odstavec",u:"Nadpis 1",v:"Nadpis 2",w:"Nadpis 3",x:"Odstranit odkaz",y:"Upravit odkaz",z:"Otevřít odkaz v nové kartě",aa:"Tento odkaz nemá žádnou URL",ab:"Textový editor, %0",ac:"Textový editor",ad:"Zpět",ae:"Znovu",af:"Uložit",ag:"Zrušit",ah:"Alternativní text",ai:"URL odkazu"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));