{% extends 'admin-layout.html' %}
{% block body %}



<div class="card">
    <div class="card-header">{{_("Why Going to Premium?")}}</div>
    <div class="card-body">
        {{_("Premium Description")}}
    </div>
</div>


{% endblock %}