(function(d){d['de']=Object.assign(d['de']||{},{a:"Datei kann nicht hochgeladen werden:",b:"Blockzitat",c:"Kursiv",d:"Fett",e:"Bild-Steuerelement",f:"Bild in voller Größe",g:"Seitenbild",h:"linksbündiges Bild",i:"zentriertes Bild",j:"rechtsbündiges Bild",k:"Bildunterschrift eingeben",l:"Bild einfügen",m:"Link",n:"Hochladen fehlgeschlagen",o:"Überschrift auswählen",p:"Überschrift",q:"Nummerierte Liste",r:"Aufzählungsliste",s:"Alternativ Text ändern",t:"Absatz",u:"Überschrift 1",v:"Überschrift 2",w:"Überschrift 3",x:"Link entfernen",y:"Link bearbeiten",z:"Link im neuen Tab öffnen",aa:"Dieser Link hat keine Adresse",ab:"Rich-Text-Editor, %0",ac:"Rich Text Editor",ad:"Rückgängig",ae:"Wiederherstellen",af:"Speichern",ag:"Abbrechen",ah:"Textalternative",ai:"Link Adresse"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));