(function(d){d['ru']=Object.assign(d['ru']||{},{a:"Невозможно загрузить файл",b:"Цитата",c:"Курсив",d:"Жирный",e:"Виджет изображений",f:"Оригинальный размер изображения",g:"Боковое изображение",h:"Выравнивание по левому краю",i:"Выравнивание по центру",j:"Выравнивание по правому краю",k:"Подпись к изображению",l:"Вставить изображение",m:"Ссылка",n:"Загрузка не выполнена",o:"Выбрать заголовок",p:"Заголовок",q:"Нумерованный список",r:"Маркированный список",s:"Редактировать альтернативный текст",t:"Параграф",u:"Заголовок 1",v:"Заголовок 2",w:"Заголовок 3",x:"Убрать ссылку",y:"Редактировать ссылку",z:"Открыть ссылку в новой вкладке",aa:"Для этой ссылки не установлен адрес URL",ab:"Редактор, %0",ac:"Редактор",ad:"Отменить",ae:"Повторить",af:"Сохранить",ag:"Отмена",ah:"Альтернативный текст",ai:"Ссылка URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));