{% extends 'flask-admin/master.html' %}
{% import 'admin/lib.html' as lib with context %}
{% import 'flask-admin/adminlte/forms.html' as forms with context %}

{% block head %}
    {{ super() }}
    {{ lib.form_css() }}
{% endblock %}

{% block body %}
    <section class="content-header">
        <h1>{{ admin_view.name }}</h1>
    </section>
    <section class="content">
        <div class="row">
            <div class="col-md-12">
                <div class="card">
                    {% block edit_form %}
                        {% set modal_name = 'Edit' + ' #' + request.args.get('id') %}
                        {{ forms.form(modal_name, icon, form, form_opts, return_url, has_more=False, is_modal=False) }}
                    {% endblock %}
                </div>
            </div>
        </div>
    </section>
{% endblock %}

{% block tail %}
    {{ super() }}
    {{ lib.form_js() }}
{% endblock %}
