{% extends 'hiddify-flask-admin/list.html' %}

{% block body %}
<div class="callout callout-info">
    {{_('domain.current_server_IPs')}}:
    {% for ip in hutils.network.get_ips()%}
    <span class="badge badge-info">{{ip}}</span>
    {%endfor%}
</div>

{{super()}}

<div class="callout callout-info">
    {{_('domain.intro')}}
</div>


<style>
    .table td {
        vertical-align: middle;
    }

    td.col-domain {
        /* direction:ltr; */
    }
</style>
{% endblock %}