(function(d){d['si']=Object.assign(d['si']||{},{a:"ගොනුව යාවත්කාලීන කළ නොහැක:",b:"Block quote",c:"ඇලකුරු",d:"තදකුරු",e:"image widget",f:"Full size image",g:"Side image",h:"Left aligned image",i:"Centered image",j:"Right aligned image",k:"Enter image caption",l:"පින්තූරය ඇතුල් කරන්න",m:"Link",n:"උඩුගත කිරීම අසාර්ථක විය",o:"Choose heading",p:"Heading",q:"අංකිත ලැයිස්තුව",r:"බුලටිත ලැයිස්තුව",s:"Change image text alternative",t:"Paragraph",u:"Heading 1",v:"Heading 2",w:"Heading 3",x:"Unlink",y:"Edit link",z:"Open link in new tab",aa:"This link has no URL",ab:"Rich Text Editor, %0",ac:"Rich Text Editor",ad:"අහෝසි කරන්න",ae:"නැවත කරන්න",af:"Save",ag:"Cancel",ah:"Text alternative",ai:"Link URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));