(function(d){d['sv']=Object.assign(d['sv']||{},{a:"Kan inte ladda upp fil:",b:"Blockcitat",c:"Kursiv",d:"Fet",e:"image widget",f:"Full size image",g:"Side image",h:"Left aligned image",i:"Centered image",j:"Right aligned image",k:"Enter image caption",l:"Infoga bild",m:"Länk",n:"Uppladdning misslyckades",o:"Välj rubrik",p:"Rubrik",q:"Numrerad lista",r:"Punktlista",s:"Change image text alternative",t:"Paragraf",u:"Rubrik 1",v:"Rubrik 2",w:"Rubrik 3",x:"Ta bort länk",y:"Redigera länk",z:"Öppna länk i ny flik",aa:"Denna länk saknar URL",ab:"Rich Text-editor, %0",ac:"Rich Text-editor",ad:"Ångra",ae:"Gör om",af:"Spara",ag:"Avbryt",ah:"Text alternative",ai:"Länkens URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));