{{_("config.dbvalues.label")}}
{{_("config.dbvalues.description")}}
{{_("config.create_easysetup_link.label")}}
{{_("config.create_easysetup_link.description")}}
{{_("config.wireguard_enable.label")}}
{{_("config.wireguard_enable.description")}}
{{_("config.wireguard_port.label")}}
{{_("config.wireguard_port.description")}}
{{_("config.wireguard_ipv6.label")}}
{{_("config.wireguard_ipv6.description")}}
{{_("config.wireguard_ipv4.label")}}
{{_("config.wireguard_ipv4.description")}}
{{_("config.wireguard_private_key.label")}}
{{_("config.wireguard_private_key.description")}}
{{_("config.wireguard_public_key.label")}}
{{_("config.wireguard_public_key.description")}}
{{_("config.wireguard_noise_trick.label")}}
{{_("config.wireguard_noise_trick.description")}}
{{_("config.ssh_server_redis_url.label")}}
{{_("config.ssh_server_redis_url.description")}}
{{_("config.ssh_server_port.label")}}
{{_("config.ssh_server_port.description")}}
{{_("config.ssh_server_enable.label")}}
{{_("config.ssh_server_enable.description")}}
{{_("config.first_setup.label")}}
{{_("config.first_setup.description")}}
{{_("config.core_type.label")}}
{{_("config.core_type.description")}}
{{_("config.warp_enable.label")}}
{{_("config.warp_enable.description")}}
{{_("config.warp_mode.label")}}
{{_("config.warp_mode.description")}}
{{_("config.warp_plus_code.label")}}
{{_("config.warp_plus_code.description")}}
{{_("config.warp_sites.label")}}
{{_("config.warp_sites.description")}}
{{_("config.dns_server.label")}}
{{_("config.dns_server.description")}}
{{_("config.reality_fallback_domain.label")}}
{{_("config.reality_fallback_domain.description")}}
{{_("config.reality_server_names.label")}}
{{_("config.reality_server_names.description")}}
{{_("config.reality_short_ids.label")}}
{{_("config.reality_short_ids.description")}}
{{_("config.reality_private_key.label")}}
{{_("config.reality_private_key.description")}}
{{_("config.reality_public_key.label")}}
{{_("config.reality_public_key.description")}}
{{_("config.reality_port.label")}}
{{_("config.reality_port.description")}}
{{_("config.special_port.label")}}
{{_("config.special_port.description")}}
{{_("config.restls1_2_domain.label")}}
{{_("config.restls1_2_domain.description")}}
{{_("config.restls1_3_domain.label")}}
{{_("config.restls1_3_domain.description")}}
{{_("config.show_usage_in_sublink.label")}}
{{_("config.show_usage_in_sublink.description")}}
{{_("config.cloudflare.label")}}
{{_("config.cloudflare.description")}}
{{_("config.license.label")}}
{{_("config.license.description")}}
{{_("config.country.label")}}
{{_("config.country.description")}}
{{_("config.package_mode.label")}}
{{_("config.package_mode.description")}}
{{_("config.utls.label")}}
{{_("config.utls.description")}}
{{_("config.telegram_bot_token.label")}}
{{_("config.telegram_bot_token.description")}}
{{_("config.is_parent.label")}}
{{_("config.is_parent.description")}}
{{_("config.parent_panel.label")}}
{{_("config.parent_panel.description")}}
{{_("config.parent_domain.label")}}
{{_("config.parent_domain.description")}}
{{_("config.parent_admin_proxy_path.label")}}
{{_("config.parent_admin_proxy_path.description")}}
{{_("config.panel_mode.label")}}
{{_("config.panel_mode.description")}}
{{_("config.log_level.label")}}
{{_("config.log_level.description")}}
{{_("config.unique_id.label")}}
{{_("config.unique_id.description")}}
{{_("config.last_hash.label")}}
{{_("config.last_hash.description")}}
{{_("config.cdn_forced_host.label")}}
{{_("config.cdn_forced_host.description")}}
{{_("config.lang.label")}}
{{_("config.lang.description")}}
{{_("config.admin_lang.label")}}
{{_("config.admin_lang.description")}}
{{_("config.admin_secret.label")}}
{{_("config.admin_secret.description")}}
{{_("config.default_useragent_string.label")}}
{{_("config.default_useragent_string.description")}}
{{_("config.use_ip_in_config.label")}}
{{_("config.use_ip_in_config.description")}}
{{_("config.tls_ports.label")}}
{{_("config.tls_ports.description")}}
{{_("config.tls_fragment_enable.label")}}
{{_("config.tls_fragment_enable.description")}}
{{_("config.tls_fragment_size.label")}}
{{_("config.tls_fragment_size.description")}}
{{_("config.tls_fragment_sleep.label")}}
{{_("config.tls_fragment_sleep.description")}}
{{_("config.tls_fragment_packets.label")}}
{{_("config.tls_fragment_packets.description")}}
{{_("config.tls_mixed_case.label")}}
{{_("config.tls_mixed_case.description")}}
{{_("config.tls_padding_enable.label")}}
{{_("config.tls_padding_enable.description")}}
{{_("config.tls_padding_length.label")}}
{{_("config.tls_padding_length.description")}}
{{_("config.tls_ech_enable.label")}}
{{_("config.tls_ech_enable.description")}}
{{_("config.mux_enable.label")}}
{{_("config.mux_enable.description")}}
{{_("config.mux_protocol.label")}}
{{_("config.mux_protocol.description")}}
{{_("config.mux_max_connections.label")}}
{{_("config.mux_max_connections.description")}}
{{_("config.mux_min_streams.label")}}
{{_("config.mux_min_streams.description")}}
{{_("config.mux_max_streams.label")}}
{{_("config.mux_max_streams.description")}}
{{_("config.mux_padding_enable.label")}}
{{_("config.mux_padding_enable.description")}}
{{_("config.mux_brutal_enable.label")}}
{{_("config.mux_brutal_enable.description")}}
{{_("config.mux_brutal_up_mbps.label")}}
{{_("config.mux_brutal_up_mbps.description")}}
{{_("config.mux_brutal_down_mbps.label")}}
{{_("config.mux_brutal_down_mbps.description")}}
{{_("config.http_ports.label")}}
{{_("config.http_ports.description")}}
{{_("config.mieru_tcp_ports.label")}}
{{_("config.mieru_tcp_ports.description")}}
{{_("config.mieru_udp_ports.label")}}
{{_("config.mieru_udp_ports.description")}}
{{_("config.kcp_ports.label")}}
{{_("config.kcp_ports.description")}}
{{_("config.kcp_enable.label")}}
{{_("config.kcp_enable.description")}}
{{_("config.decoy_domain.label")}}
{{_("config.decoy_domain.description")}}
{{_("config.dnstt_enable.label")}}
{{_("config.dnstt_enable.description")}}
{{_("config.dnstt_resolvers.label")}}
{{_("config.dnstt_resolvers.description")}}
{{_("config.proxy_path.label")}}
{{_("config.proxy_path.description")}}
{{_("config.proxy_path_admin.label")}}
{{_("config.proxy_path_admin.description")}}
{{_("config.proxy_path_client.label")}}
{{_("config.proxy_path_client.description")}}
{{_("config.firewall.label")}}
{{_("config.firewall.description")}}
{{_("config.netdata.label")}}
{{_("config.netdata.description")}}
{{_("config.http_proxy_enable.label")}}
{{_("config.http_proxy_enable.description")}}
{{_("config.block_iran_sites.label")}}
{{_("config.block_iran_sites.description")}}
{{_("config.allow_invalid_sni.label")}}
{{_("config.allow_invalid_sni.description")}}
{{_("config.auto_update.label")}}
{{_("config.auto_update.description")}}
{{_("config.speed_test.label")}}
{{_("config.speed_test.description")}}
{{_("config.only_ipv4.label")}}
{{_("config.only_ipv4.description")}}
{{_("config.shared_secret.label")}}
{{_("config.shared_secret.description")}}
{{_("config.telegram_enable.label")}}
{{_("config.telegram_enable.description")}}
{{_("config.telegram_adtag.label")}}
{{_("config.telegram_adtag.description")}}
{{_("config.telegram_lib.label")}}
{{_("config.telegram_lib.description")}}
{{_("config.telegram_fakedomain.label")}}
{{_("config.telegram_fakedomain.description")}}
{{_("config.v2ray_enable.label")}}
{{_("config.v2ray_enable.description")}}
{{_("config.torrent_block.label")}}
{{_("config.torrent_block.description")}}
{{_("config.tuic_enable.label")}}
{{_("config.tuic_enable.description")}}
{{_("config.tuic_port.label")}}
{{_("config.tuic_port.description")}}
{{_("config.hysteria_enable.label")}}
{{_("config.hysteria_enable.description")}}
{{_("config.hysteria_port.label")}}
{{_("config.hysteria_port.description")}}
{{_("config.hysteria_obfs_enable.label")}}
{{_("config.hysteria_obfs_enable.description")}}
{{_("config.hysteria_up_mbps.label")}}
{{_("config.hysteria_up_mbps.description")}}
{{_("config.hysteria_down_mbps.label")}}
{{_("config.hysteria_down_mbps.description")}}
{{_("config.shadowsocks2022_enable.label")}}
{{_("config.shadowsocks2022_enable.description")}}
{{_("config.shadowsocks2022_method.label")}}
{{_("config.shadowsocks2022_method.description")}}
{{_("config.shadowsocks2022_port.label")}}
{{_("config.shadowsocks2022_port.description")}}
{{_("config.ssfaketls_enable.label")}}
{{_("config.ssfaketls_enable.description")}}
{{_("config.ssfaketls_fakedomain.label")}}
{{_("config.ssfaketls_fakedomain.description")}}
{{_("config.shadowtls_enable.label")}}
{{_("config.shadowtls_enable.description")}}
{{_("config.shadowtls_fakedomain.label")}}
{{_("config.shadowtls_fakedomain.description")}}
{{_("config.ssr_enable.label")}}
{{_("config.ssr_enable.description")}}
{{_("config.ssr_fakedomain.label")}}
{{_("config.ssr_fakedomain.description")}}
{{_("config.vmess_enable.label")}}
{{_("config.vmess_enable.description")}}
{{_("config.domain_fronting_domain.label")}}
{{_("config.domain_fronting_domain.description")}}
{{_("config.domain_fronting_http_enable.label")}}
{{_("config.domain_fronting_http_enable.description")}}
{{_("config.domain_fronting_tls_enable.label")}}
{{_("config.domain_fronting_tls_enable.description")}}
{{_("config.ws_enable.label")}}
{{_("config.ws_enable.description")}}
{{_("config.grpc_enable.label")}}
{{_("config.grpc_enable.description")}}
{{_("config.httpupgrade_enable.label")}}
{{_("config.httpupgrade_enable.description")}}
{{_("config.xhttp_enable.label")}}
{{_("config.xhttp_enable.description")}}
{{_("config.naive_enable.label")}}
{{_("config.naive_enable.description")}}
{{_("config.naive_port.label")}}
{{_("config.naive_port.description")}}
{{_("config.mieru_enable.label")}}
{{_("config.mieru_enable.description")}}
{{_("config.mieru_multiplexing.label")}}
{{_("config.mieru_multiplexing.description")}}
{{_("config.mieru_handshake.label")}}
{{_("config.mieru_handshake.description")}}
{{_("config.vless_enable.label")}}
{{_("config.vless_enable.description")}}
{{_("config.trojan_enable.label")}}
{{_("config.trojan_enable.description")}}
{{_("config.reality_enable.label")}}
{{_("config.reality_enable.description")}}
{{_("config.tcp_enable.label")}}
{{_("config.tcp_enable.description")}}
{{_("config.quic_enable.label")}}
{{_("config.quic_enable.description")}}
{{_("config.xtls_enable.label")}}
{{_("config.xtls_enable.description")}}
{{_("config.h2_enable.label")}}
{{_("config.h2_enable.description")}}
{{_("config.db_version.label")}}
{{_("config.db_version.description")}}
{{_("config.last_priodic_usage_check.label")}}
{{_("config.last_priodic_usage_check.description")}}
{{_("config.branding_title.label")}}
{{_("config.branding_title.description")}}
{{_("config.branding_site.label")}}
{{_("config.branding_site.description")}}
{{_("config.branding_freetext.label")}}
{{_("config.branding_freetext.description")}}
{{_("config.not_found.label")}}
{{_("config.not_found.description")}}
{{_("config.path_vmess.label")}}
{{_("config.path_vmess.description")}}
{{_("config.path_vless.label")}}
{{_("config.path_vless.description")}}
{{_("config.path_trojan.label")}}
{{_("config.path_trojan.description")}}
{{_("config.path_naive.label")}}
{{_("config.path_naive.description")}}
{{_("config.path_v2ray.label")}}
{{_("config.path_v2ray.description")}}
{{_("config.path_ss.label")}}
{{_("config.path_ss.description")}}
{{_("config.path_xhttp.label")}}
{{_("config.path_xhttp.description")}}
{{_("config.path_httpupgrade.label")}}
{{_("config.path_httpupgrade.description")}}
{{_("config.path_ws.label")}}
{{_("config.path_ws.description")}}
{{_("config.path_tcp.label")}}
{{_("config.path_tcp.description")}}
{{_("config.path_grpc.label")}}
{{_("config.path_grpc.description")}}
{{_("config.sub_full_singbox_enable.label")}}
{{_("config.sub_full_singbox_enable.description")}}
{{_("config.sub_singbox_ssh_enable.label")}}
{{_("config.sub_singbox_ssh_enable.description")}}
{{_("config.sub_full_xray_json_enable.label")}}
{{_("config.sub_full_xray_json_enable.description")}}
{{_("config.sub_full_links_enable.label")}}
{{_("config.sub_full_links_enable.description")}}
{{_("config.sub_full_links_b64_enable.label")}}
{{_("config.sub_full_links_b64_enable.description")}}
{{_("config.sub_full_clash_enable.label")}}
{{_("config.sub_full_clash_enable.description")}}
{{_("config.sub_full_clash_meta_enable.label")}}
{{_("config.sub_full_clash_meta_enable.description")}}
{{_("config.ssh_host_rsa_pk.label")}}
{{_("config.ssh_host_rsa_pk.description")}}
{{_("config.ssh_host_rsa_pub.label")}}
{{_("config.ssh_host_rsa_pub.description")}}
{{_("config.ssh_host_ed25519_pk.label")}}
{{_("config.ssh_host_ed25519_pk.description")}}
{{_("config.ssh_host_ed25519_pub.label")}}
{{_("config.ssh_host_ed25519_pub.description")}}
{{_("config.ssh_host_ecdsa_pk.label")}}
{{_("config.ssh_host_ecdsa_pk.description")}}
{{_("config.ssh_host_ecdsa_pub.label")}}
{{_("config.ssh_host_ecdsa_pub.description")}}
{{_("config.ssh_host_dsa_pk.label")}}
{{_("config.ssh_host_dsa_pk.description")}}
{{_("config.ssh_host_dsa_pub.label")}}
{{_("config.ssh_host_dsa_pub.description")}}
{{_("config.hiddifycli_enable.label")}}
{{_("config.hiddifycli_enable.description")}}

{{_("config.admin.label")}}{{_("config.admin.description")}}
{{_("config.branding.label")}}{{_("config.branding.description")}}
{{_("config.general.label")}}{{_("config.general.description")}}
{{_("config.proxies.label")}}{{_("config.proxies.description")}}
{{_("config.domain_fronting.label")}}{{_("config.domain_fronting.description")}}
{{_("config.telegram.label")}}{{_("config.telegram.description")}}
{{_("config.http.label")}}{{_("config.http.description")}}
{{_("config.tls.label")}}{{_("config.tls.description")}}
{{_("config.mux.label")}}{{_("config.mux.description")}}
{{_("config.tls_trick.label")}}{{_("config.tls_trick.description")}}
{{_("config.ssh.label")}}{{_("config.ssh.description")}}
{{_("config.ssfaketls.label")}}{{_("config.ssfaketls.description")}}
{{_("config.mieru.label")}}{{_("config.mieru.description")}}
{{_("config.shadowtls.label")}}{{_("config.shadowtls.description")}}
{{_("config.restls.label")}}{{_("config.restls.description")}}
{{_("config.tuic.label")}}{{_("config.tuic.description")}}
{{_("config.hysteria.label")}}{{_("config.hysteria.description")}}
{{_("config.ssr.label")}}{{_("config.ssr.description")}}
{{_("config.kcp.label")}}{{_("config.kcp.description")}}
{{_("config.hidden.label")}}{{_("config.hidden.description")}}
{{_("config.advanced.label")}}{{_("config.advanced.description")}}
{{_("config.too_advanced.label")}}{{_("config.too_advanced.description")}}
{{_("config.warp.label")}}{{_("config.warp.description")}}
{{_("config.reality.label")}}{{_("config.reality.description")}}
{{_("config.wireguard.label")}}{{_("config.wireguard.description")}}
{{_("config.shadowsocks.label")}}{{_("config.shadowsocks.description")}}

{{_("direct")}}
{{_("sub_link_only")}}
{{_("cdn")}}
{{_("auto_cdn_ip")}}
{{_("relay")}}
{{_("worker")}}
{{_("fake")}}
{{_("reality")}}
{{_("special_reality_tcp")}}
{{_("special_reality_xhttp")}}
{{_("special_reality_grpc")}}
{{_("old_xtls_direct")}}
{{_("dnstt")}}

{{_("no_reset")}}
{{_("monthly")}}
{{_("weekly")}}
{{_("daily")}}

{{_("config.panel_mode.standalone")}}
{{_("config.panel_mode.parent")}}
{{_("config.panel_mode.child")}}

{{_("config.log_level.TRACE")}}
{{_("config.log_level.DEBUG")}}
{{_("config.log_level.INFO")}}
{{_("config.log_level.SUCCESS")}}
{{_("config.log_level.WARNING")}}
{{_("config.log_level.ERROR")}}
{{_("config.log_level.CRITICAL")}}

{{_("config.lang.en")}}
{{_("config.lang.fa")}}
{{_("config.lang.ru")}}
{{_("config.lang.pt")}}
{{_("config.lang.zh")}}
{{_("config.lang.my")}}

{{_("config.admin_lang.en")}}
{{_("config.admin_lang.fa")}}
{{_("config.admin_lang.ru")}}
{{_("config.admin_lang.pt")}}
{{_("config.admin_lang.zh")}}
{{_("config.admin_lang.my")}}

{{_("config.mieru_multiplexing.MULTIPLEXING_DEFAULT")}}
{{_("config.mieru_multiplexing.MULTIPLEXING_LOW")}}
{{_("config.mieru_multiplexing.MULTIPLEXING_MIDDLE")}}
{{_("config.mieru_multiplexing.MULTIPLEXING_HIGH")}}

{{_("config.mieru_handshake.HANDSHAKE_DEFAULT")}}
{{_("config.mieru_handshake.HANDSHAKE_NO_WAIT")}}
{{_("config.mieru_handshake.HANDSHAKE_STANDARD")}}

