(function(d){d['hr']=Object.assign(d['hr']||{},{a:"Datoteku nije moguće poslati:",b:"Blok citat",c:"Ukošeno",d:"Podebljano",e:"Slika widget",f:"Slika pune veličine",g:"Slika sa strane",h:"Lijevo poravnata slika",i:"Centrirana slika",j:"Slika poravnata desno",k:"Unesite naslov slike",l:"Umetni sliku",m:"Veza",n:"Slanje nije uspjelo",o:"Odaberite naslov",p:"Naslov",q:"Brojčana lista",r:"Obična lista",s:"Promijeni alternativni tekst slike",t:"Paragraf",u:"Naslov 1",v:"Naslov 2",w:"Naslov 3",x:"Ukloni vezu",y:"Uredi vezu",z:"Otvori vezu u novoj kartici",aa:"Ova veza nema URL",ab:"Rich Text Editor, %0",ac:"Rich Text Editor",ad:"Poništi",ae:"Ponovi",af:"Snimi",ag:"Poništi",ah:"Alternativni tekst",ai:"URL veze"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));