
<div class="col">
  {{_("Please select one of the following applications:")}}
      <details  data-widget="collapse">
        <summary class="callout callout-success">
          {{_("user.home.ios.stash.title")}} 
          
      </summary>
      <br>
      {{_("user.home.ios.stash.intro")}}
      <br>
      {% with show_meta=True, ios=True %}
      {% include "home/clash-links.html" %}
      {% endwith %}
        
</details>

  <details  data-widget="collapse">
    <summary class="callout callout-success">
      
      {{_("user.home.ios.others.title")}} <button class='btn btn-sm btn-default' onclick='show_video("ios-fair")'><i class="fa-solid fa-question"></i></button>  
      </summary>

      
      {{_("user.home.ios.others.intro")}}
      <br>
      <div class="btn-group">
        <a href="https://{{domain}}/{{hconfigs[ConfigEnum.proxy_path]}}/{{user.uuid}}/all.txt?name={{db_domain.alias or db_domain.domain}}&asn={{asn}}&mode={{mode}}&base64=True"
          class="btn btn-light orig-link"> {{_('user.home.allconfig.link-configs')}} b64</a>
      </div>
      <br>
    </details>
    
</div>
<!-- /.col -->