{# extends 'master.html' #}
{%block body%}
<section class="content">
    <div class="error-page">
        <h2 class="headline text-danger">500</h2>
        <div class="error-content">
            <h3><i class="fas fa-exclamation-triangle text-danger"></i> {{_('Oops! Something went wrong.')}} Hiddify
                Panel {{version}}</h3>
            {% if has_update %}
            <h3> {{_('outdated_panel')}} </h3>
            {% else %}
            <p>
                {{_('Please create an issue on Github.')}}
            <div style="width: 100%;height: 50px;">
                <a href="{{issue_link}}">
                    <input type="button" value="Create Issue on Github">
                </a>
            </div>
            </p>
            {% endif %}

        </div>
    </div>
    {% if not has_update %}
    <pre class="ltr">
        {%if "codec can't encode characters" in (""~error) %}
        Please restart the server.
        {%endif%}
{{error}}
{{trace}}
                    </pre>
    {% endif %}

</section>
{%endblock%}