{# extends 'master.html' #}
{%block body%}
<section class="content">
    <div class="error-page">
        <h2 class="headline text-danger">{{error.status_code}} {{error.message}}</h2>
        <div class="error-content">
            <!-- <h3><i class="fas fa-exclamation-triangle text-danger"></i> {{_('Oops! Something went wrong.')}}</h3> -->
            <p>
                {{error.detail if error.detail}}
            </p>

        </div>
    </div>

</section>
{%endblock%}