{{_("user.home.tool.doh.title")}}




<div class="btn-group">
    <a href="https://{{domain}}/{{hconfigs[ConfigEnum.proxy_path]}}/dns/dns-query"
        class="btn btn-primary copy-link">{{_('copy')}}</a>
</div>
<pre dir="ltr" style="text-align: left; font: arial">
    https://{{domain}}/{{hconfigs[ConfigEnum.proxy_path]}}/dns/dns-query
    </pre>
{{_("user.home.tool.doh.help")}}

