(function(d){d['es']=Object.assign(d['es']||{},{a:"No se pudo cargar el archivo:",b:"Entrecomillado",c:"Cursiva",d:"Negrita",e:"Widget de imagen",f:"Imagen a tamaño completo",g:"Imagen lateral",h:"Imagen alineada a la izquierda",i:"Imagen centrada",j:"Imagen alineada a la derecha",k:"Introducir título de la imagen",l:"Insertar imagen",m:"Enlace",n:"Fallo en la subida",o:"Elegir Encabezado",p:"Encabezado",q:"Lista numerada",r:"Lista de puntos",s:"Cambiar el texto alternativo de la imagen",t:"Párrafo",u:"Encabezado 1",v:"Encabezado 2",w:"Encabezado 3",x:"Quitar enlace",y:"Editar enlace",z:"Abrir enlace en una pestaña nueva",aa:"Este enlace no tiene URL",ab:"Editor de Texto Enriquecido, %0",ac:"Editor de Texto Enriquecido",ad:"Deshacer",ae:"Rehacer",af:"Guardar",ag:"Cancelar",ah:"Texto alternativo",ai:"URL del enlace"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));