{% from 'bootstrap4/form.html' import render_form %}

{% extends 'admin-layout.html' %}

{% block body %}
{{super()}}



<div class="card">
    <div class="card-header">{{_("admin.proxy.global_config")}}</div>
    <div class="card-body">
        {% if 1 or hconfig(ConfigEnum.license) %}
        <a class="btn btn-info" href="{{hurl_for('flask.proxy.index_view')}}">{{_("Edit Proxy Names")}}</a>
        {% endif %}
        <div class="row">
            <div class="col-12 global-form">
                {{render_form(global_config_form,extra_classes="card-columns")}}
            </div>
        </div>
    </div>
</div>
<div class="card">
    <div class="card-header">{{_("admin.proxy.detailed_config")}}</div>
    <div class="card-body">
        <div class="row">
            <div class="col-12">
                {{render_form(detailed_config_form,extra_classes="card-columns")}}
            </div>
        </div>
    </div>
</div>

<style>
    fieldset {
        background-color: initial;
        background-clip: border-box;
        border: 1 solid rgba(0, 0, 0, 0.125);
        border-radius: 0.25rem;
        box-shadow: 0 0 1px rgb(0 0 0 / 13%), 0 1px 3px rgb(0 0 0 / 20%);
        margin-bottom: 1rem;
        padding: 10px;
        display: inline-block;
        width: 100%;
    }

    legend {
        background-color: initial;
        margin: 0px;
    }

    /* legend+div>label {
        display: none;
    } */

    @media (min-width: 576px) {
        .card-columns {
            column-count: 1;
        }
    }

    @media (min-width: 768px) {
        .card-columns {
            column-count: 2;
        }
    }

    .global-form .custom-switch {
        display: inline-block;
    }

    #submit_global {
        display: block;
    }
</style>
{% endblock %}