<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="refresh" content="0;url={{url}}">
    <title>Redirecting...</title>
    <script>
        // JavaScript fallback in case the meta tag doesn't work
        
        window.location.href = "{{url}}";
        
    </script>
</head>
<body>
    <p>If you are not redirected, <a href="{{url}}">click here</a>.</p>
</body>
</html>
