(function(d){d['ro']=Object.assign(d['ro']||{},{a:"Nu pot încărca fișierul:",b:"Bloc citat",c:"Oblic",d:"Îngroșat",e:"widget imagine",f:"Imagine mărime completă",g:"Imagine laterală",h:"Imagine aliniată stângă",i:"Imagine aliniată pe centru",j:"Imagine aliniată dreapta",k:"Introdu titlul descriptiv al imaginii",l:"Inserează imagine",m:"Link",n:"Încărcare eșuată",o:"Alege titlu rubrică",p:"Titlu rubrică",q:"Listă numerotată",r:"Listă cu puncte",s:"Schimbă textul alternativ al imaginii",t:"Paragraf",u:"Titlu rubrică 1",v:"Titlu rubrică 2",w:"Titlu rubrică 3",x:"Șterge link",y:"Edit link",z:"Open link in new tab",aa:"This link has no URL",ab:"Editor de text îmbunătățit, %0",ac:"Editor de text îmbunătățit",ad:"Anulează",ae:"Revenire",af:"Salvare",ag:"Anulare",ah:"Text alternativ",ai:"Link URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));