(function(d){d['sq']=Object.assign(d['sq']||{},{a:"Nuk mund të ngarkojë skedën:",b:"Thonjëzat",c:"Pjerrtë",d:"Trash",e:"Vegla e fotos",f:"Foto me madhësi të plotë",g:"Foto anësore",h:"Foto e vendosur majtas",i:"Foto e vendosur në mes",j:"Foto e vendosur djathtas",k:"Shto përshkrimin e fotos",l:"Shto Foto",m:"Shto Nyjën",n:"Ngarkimi dishtoi",o:"Përzgjidh nëntitullin",p:"Nëntitulli",q:"Listë me Numra",r:"Listë me Pika",s:"Ndrysho tekstin alternativ të fotos",t:"Paragrafi",u:"Nëntitulli 1",v:"Nëntitulli 2",w:"Nëntitulli 3",x:"Largo Nyjën",y:"Edit link",z:"Open link in new tab",aa:"This link has no URL",ab:"Redaktues i Tekstit të Pasur, %0",ac:"Redaktues i Tekstit të Pasur",ad:"Rikthe",ae:"Ribëje",af:"Ruaje",ag:"Anulo",ah:"Teksti alternativ",ai:"Link URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));